//Zdalna migawka dla Nikona/Canona - rShutter, copyright Robert Wogajew 2023
//28.01.2023: uproszenie eliminacji drga stykw i warunkw wysyania ramki danych
//27.03.2023:  wprowadzono zmiany do ramki Nikona. GOTOWY - sprawdzony na ukdzie prototypowym
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <inttypes.h>
#include <string.h>
#include <avr/sleep.h>
#include "defines.h"

void sendNikon(void);
void sendCanon(void);

//#############################################################################################
//#############################################################################################

int main(void)
{
		//Redukcja poboru mocy przez wyczenie komparatora analogowego
		ACSR |= (1<<ACD);
		//Redukcja poboru mocy przez wyczenie zasilania przetwornika ADC
		PRR = (1<<PRADC);
		//Uruchomienie przerwania Pin Change Interrupt (wybudzanie procesora w reakcji na nacinicie switcha SHOT)
		PCICR = (1<<PCIE0);
		//Konfiguracja przerwania, jak wyej. Zmiana stanu wyprowadzenia PB1/PCINT1 wyzwala przerwanie (i wybudza procesor)
		PCMSK = (1<<PCINT1);
		//Podcignicie portu wyzwolenia migawki (SHOT -> PB1/PCINT1) pod VCC
		SHOT_PULL_UP_ON;
		//Port sterujcy diod IR (PB0), jako wyjciowy (z domylnym stanem 0)
		IR_AS_OUTPUT;
		//Konfiguracja Timera0 odpowiedzialnego za generowanie przebiegu 38.4 kHz (czstotliwoci nonej) na wyjciu diody IR
		TCCR0B = (1<<WGM02);  //Tryb CTC
		OCR0A = 12;  //38.4 kHz


		while(1)
		{
				cli();
				set_sleep_mode(SLEEP_MODE_PWR_DOWN);
				sleep_enable();
				sei();
				sleep_cpu();
				//W tym miejscu pimy i czekamy na wybudzenie przez zmian stanu przycisku SHOT
				sleep_disable();

				//Podcigamy przycisk MODE pod VCC
				MODE_PULL_UP_ON;

				//Sprawdzamy stan przycisku MODE, eby zdecydowa dla jakiego systemu (NIKON/CANON) wysyany bdzie sygna IR. Nastpnie
				//wysyamy stosown ramk poprzez diod IR. Maksymalny czas wykonania (Nikon) 135 ms, w tym 3.2 ms zaczenia diody IR
				if(MODE_IS_NIKON) sendNikon(); else sendCanon();

				//Wyczamy podcignicie przycisku MODE pod VCC, gdy zwarty do masy (tryb Nikon) pobieraby dodatkowy prd w trybie Power Down
				MODE_PULL_UP_OFF;

				//Prosta eliminacja drgania stykw
				_delay_ms(35);

				//Czekamy na ewentualne zwolnienie przycisku SHOT
				while(SHOT_IS_PRESSED);

				//Prosta eliminacja drgania stykw
				_delay_ms(35);

				//Wyczyszczenie flagi przerwania Pin Change Interrupt, eby bez potrzeby (potencjalne drgania stykw) nie zostao od razu wywoane ponownie
				PCIFR = (1<<PCIF0);
				//Uruchomienie przerwania Pin Change Interrupt (wybudzanie procesora w reakcji na nacinicie switcha SHOT)
				PCICR = (1<<PCIE0);

		}
}

//#############################################################################################
//#############################################################################################
//Czas wykonania: 135ms, w tym zaczenie diody IR: 3.2 ms (przy wypenieniu 50%)

void sendNikon(void)
{
		//Dwukrotne wysanie ramki z odstpem zdefiniowanym standardem Nikon (generowanie nonej na wyjciu sterujcym diod IR)
		for(uint8_t i=0; i<2; i++)
		{
				BURST_FUNCT_ON;
				BURST_START;
				_delay_ms(START_PULSE);
				BURST_STOP;
				BURST_FUNCT_OFF;
				_delay_ms(START_PAUSE);
				BURST_FUNCT_ON;
				BURST_START;
				_delay_ms(DATA_PULSE);
				BURST_STOP;
				BURST_FUNCT_OFF;
				_delay_ms(FIRST_PAUSE);
				BURST_FUNCT_ON;
				BURST_START;
				_delay_ms(DATA_PULSE);
				BURST_STOP;
				BURST_FUNCT_OFF;
				_delay_ms(SECOND_PAUSE);
				BURST_FUNCT_ON;
				BURST_START;
				_delay_ms(DATA_PULSE);
				BURST_STOP;
				BURST_FUNCT_OFF;

				if(i == 0) _delay_ms(NIKON_FRAME_PAUSE);
		}
}

//#############################################################################################
//Czas wykonania: 8.3 ms, w tym zaczenie diody IR: 0.48 ms  (przy wypenieniu 50%)

void sendCanon(void)
{
		//Dwukrotne wysanie ramki z odstpem zdefiniowanym standardem Canon
		for(uint8_t i=0; i<2; i++)
		{
				for(uint8_t i=0; i<PULSE_NR; i++)
				{
						IR_ON;
						_delay_us(PULSE_LENGTH);
						IR_OFF;
						_delay_us(PULSE_LENGTH);
				}

				if(i == 0) _delay_ms(CANON_FRAME_PAUSE);
		}
}

//#############################################################################################
//#############################################################################################
//Przerwanie Pin Change Interrupt (wybudzanie procesora w reakcji na nacinicie switcha SHOT)

ISR(PCINT0_vect)
{
		//Tymczasowe wyczenie przerwania Pin Change Interrupt, by nie zgaszao si wielokrotnie na skutek drga stykw
		PCICR = 0x00;
}

//#############################################################################################
//#############################################################################################
